package org.vraptor.component;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;
import org.picocontainer.Characteristics;
import org.picocontainer.DefaultPicoContainer;
import org.picocontainer.MutablePicoContainer;
import org.vraptor.ValidationErrorsFactory;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.scope.ApplicationContext;
import org.vraptor.scope.DefaultApplicationContext;
import org.vraptor.url.DefaultLogicLocator;
import org.vraptor.url.DefaultViewLocator;
import org.vraptor.url.LogicLocator;
import org.vraptor.url.ViewLocator;
import org.vraptor.validator.DefaultValidationErrorsFactory;
import org.vraptor.view.RegexViewManager;
import org.vraptor.view.ViewManager;
import org.vraptor.webapp.DefaultComponentManager;

public class DefaultComponentContainer implements ComponentContainer {

    private final MutablePicoContainer container;

    private final static Logger LOG = Logger.getLogger(DefaultComponentContainer.class);

    private final static Class<?>[] TYPES = new Class<?>[]{ValidationErrorsFactory.class, 
    	LogicLocator.class, ComponentManager.class, LogicMethodFactory.class, 
    	ParameterInfoProvider.class, ViewLocator.class, ViewManager.class};

    public DefaultComponentContainer(ServletContext context) {
        this();
        container.addComponent(context);
        for (Class<?> type : TYPES) {
            String found = context.getInitParameter(type.getName());
            if (found != null) {
                try {
                    LOG.debug("Replacing default implementation of " + type.getName() + " with " + found);
                    container.removeComponent(type);
                    container.as(Characteristics.CACHE).addComponent(type, Class.forName(found));
                } catch (ClassNotFoundException e) {
                    LOG.error("Unable to load component implementation", e);
                }
            }
        }
    }

    public DefaultComponentContainer() {
        this.container = new DefaultPicoContainer();
        container.as(Characteristics.CACHE).addComponent(ValidationErrorsFactory.class, DefaultValidationErrorsFactory.class);
        container.as(Characteristics.CACHE).addComponent(ParameterInfoProvider.class, DefaultParameterInfoProvider.class);
        container.as(Characteristics.CACHE).addComponent(LogicMethodFactory.class, DefaultLogicMethodFactory.class);
        container.as(Characteristics.CACHE).addComponent(LogicLocator.class, DefaultLogicLocator.class);
        container.as(Characteristics.CACHE).addComponent(ComponentManager.class, DefaultComponentManager.class);
        container.as(Characteristics.CACHE).addComponent(ApplicationContext.class, DefaultApplicationContext.class);
        container.as(Characteristics.CACHE).addComponent(ViewLocator.class, DefaultViewLocator.class);
        container.as(Characteristics.CACHE).addComponent(ReflectionUtil.class, ReflectionUtil.class);
        container.as(Characteristics.CACHE).addComponent(ViewManager.class, RegexViewManager.class);
    }

    public ValidationErrorsFactory getValidationErrorsFactory() {
        return (ValidationErrorsFactory) container.getComponent(ValidationErrorsFactory.class);
    }

    public LogicLocator getLogicLocator() {
        return (LogicLocator) container.getComponent(LogicLocator.class);
    }

    public ComponentManager getComponentManager() {
        return (ComponentManager) container.getComponent(ComponentManager.class);
    }

    public ApplicationContext getApplicationContext() {
        return (ApplicationContext) container.getComponent(ApplicationContext.class);
    }

    public ViewLocator getViewLocator() {
        return (ViewLocator) container.getComponent(ViewLocator.class);
    }

	public ViewManager getViewManager() {
		return (ViewManager) container.getComponent(ViewManager.class);
	}

}

