package org.vraptor.component;

import java.lang.reflect.Type;

import org.vraptor.converter.Converter;
import org.vraptor.introspector.ReadParameter;
import org.vraptor.reflection.SettingException;

/**
 * Defines an argument that a logic method receives.
 *
 * @author Guilherme Silveira
 * @since 2.3
 */
public class MethodParameter implements ReadParameter {

	private final Clazz paramType;

	private final String key;

	private final int position;

	private final Type generic;

	public MethodParameter(Class<?> param, Type generic, int position, String key) {
		this.paramType = new Clazz(param);
		this.position = position;
		this.generic = generic;
		this.key = key;
	}

	public String getKey() {
		return key;
	}

	public Class<? extends Converter> getOverridenConverter() {
		return null;
	}

	public Object guaranteeExistence(Object component, Object methodParams[]) throws SettingException {
		return methodParams[position];
	}

	public boolean mightCreate() {
		return true;
	}

	public Class<?> getType() {
		return paramType.getType();
	}

	public void set(Object component, Object[] methodParams, Object value) throws SettingException {
		methodParams[position] = value;
	}

	public Type getGenericType() {
		return generic;
	}

	public Object newInstance() throws ComponentInstantiationException {
		return paramType.newInstance();
	}

	public int getPosition() {
		return this.position;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((generic == null) ? 0 : generic.hashCode());
		result = prime * result + ((key == null) ? 0 : key.hashCode());
		result = prime * result + ((paramType == null) ? 0 : paramType.hashCode());
		result = prime * result + position;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MethodParameter other = (MethodParameter) obj;
		if (generic == null) {
			if (other.generic != null)
				return false;
		}
		else if (!generic.equals(other.generic))
			return false;
		if (key == null) {
			if (other.key != null)
				return false;
		}
		else if (!key.equals(other.key))
			return false;
		if (paramType == null) {
			if (other.paramType != null)
				return false;
		}
		else if (!paramType.equals(other.paramType))
			return false;
		if (position != other.position)
			return false;
		return true;
	}
	
	@Override
	public String toString() {
		return String.format("(MethodParameter param:%s, generic:%s, position:%d, key:%s)",
				paramType, generic, position, key);
	}

}
