package org.vraptor.core;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicRequest;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.http.VRaptorServletRequest;
import org.vraptor.http.VRaptorServletResponse;
import org.vraptor.interceptor.ComponentLookupInterceptor;
import org.vraptor.interceptor.ExecuteLogicInterceptor;
import org.vraptor.interceptor.FlashScopeInterceptor;
import org.vraptor.interceptor.InjectionInterceptor;
import org.vraptor.interceptor.InterceptorDealer;
import org.vraptor.interceptor.InterceptorType;
import org.vraptor.interceptor.OutjectionInterceptor;
import org.vraptor.interceptor.RegisterAttributesInteceptor;
import org.vraptor.interceptor.SettingAndValidationInterceptor;
import org.vraptor.interceptor.ViewInterceptor;
import org.vraptor.scope.DefaultLogicRequest;
import org.vraptor.url.DefaultRequestInfo;
import org.vraptor.url.InvalidURLException;
import org.vraptor.url.RequestInfo;
import org.vraptor.view.ViewException;
import org.vraptor.view.ViewManager;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;

/**
 * VRaptor's engine.
 *
 * @author Guilherme Silveira
 * @author Paulo Silveira
 */
@SuppressWarnings("unchecked")
public class VRaptorExecution {

	private static final Logger LOG = Logger.getLogger(VRaptorExecution.class);

	private final Controller controller;

    private final ComponentType componentType;
    private final LogicMethod method;

    private final WebRequest webRequest;

    private final static Class<Interceptor>[] AFTER = new Class[] { ComponentLookupInterceptor.class,
			InjectionInterceptor.class, SettingAndValidationInterceptor.class, ExecuteLogicInterceptor.class,
			OutjectionInterceptor.class, ViewInterceptor.class };

	public VRaptorExecution(ComponentType componentType, LogicMethod method, Controller controller,
            WebRequest webRequest) {
        this.componentType = componentType;
        this.method = method;
        this.controller = controller;
        this.webRequest = webRequest;
    }

	public String execute() throws ComponentNotFoundException, LogicNotFoundException, ViewException, LogicException,
            InvalidURLException {

		if (LOG.isDebugEnabled()) {
			LOG.debug("Calling execute on " + webRequest + " / " + method.toString());
		}


        RequestInfo info = new DefaultRequestInfo(componentType.getName(), method.getName());
        DefaultLogicDefinition definition = new DefaultLogicDefinition(componentType, method);
		LogicRequest logicRequest = new DefaultLogicRequest(info, webRequest, definition);
		ViewManager viewManager = controller.getWebApplication().getContainer().getViewLocator()
                .locate(webRequest.getRequest(), method, controller.getWebApplication().getDefaultViewManager());

        webRequest.getRequest().setAttribute(ViewManager.class.getName(), viewManager);
        webRequest.getRequest().setAttribute("context", logicRequest);
		webRequest.getRequest().setAttribute("application", controller.getWebApplication().getApplicationContext());

		List<Interceptor> before = new ArrayList<Interceptor>();
		before.add(new RegisterAttributesInteceptor());
        before.add(new FlashScopeInterceptor());

		List<InterceptorType> fullInterceptors = new ArrayList<InterceptorType>(componentType.getInterceptors());
		for (Class<? extends Interceptor> type : AFTER) {
			fullInterceptors.add(InterceptorType.getType(type));
		}
		InterceptorDealer dealer = new InterceptorDealer(logicRequest, controller.getIntrospector(), before,
				fullInterceptors, new ArrayList<Interceptor>());
		InterceptorsLogicFlow logicFlow = new InterceptorsLogicFlow(logicRequest, dealer);

		logicFlow.execute();

		return logicFlow.getLogicRequest().getResult();
	}

}
