package org.vraptor.http;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.vraptor.VRaptorException;
import org.vraptor.component.ComponentContainer;
import org.vraptor.config.ConfigException;
import org.vraptor.core.Controller;
import org.vraptor.core.DefaultController;
import org.vraptor.introspector.Introspector;
import org.vraptor.view.ViewManager;
import org.vraptor.webapp.DefaultWebApplication;
import org.vraptor.webapp.WebApplication;

/**
 * VRaptor's dispatcher factory.
 *
 * @author Guilherme Silveira
 */
public class DispatcherFactory {

	private static final Logger LOG = Logger.getLogger(DispatcherFactory.class);
	
	public Dispatcher configure(ServletContext servletContext, ComponentContainer componentContainer) throws ServletException {
		WebApplication app = new DefaultWebApplication(servletContext, componentContainer);
		return configureDispatcher(app);
	}
	
	public Dispatcher configure(ServletContext servletContext) throws ServletException {
		WebApplication app = new DefaultWebApplication(servletContext);
		return configureDispatcher(app);
	}
	
	private Dispatcher configureDispatcher(WebApplication app) throws ServletException {
		try {
			Controller controller = new DispatcherFactory().configureController(app);
			return new DefaultDispatcher(controller);
		}
		catch (ConfigException e) {
			throw new ServletException("Unable to configure vraptor2", e);
		}
	}

	private Controller configureController(WebApplication app) throws ConfigException {
		LOG.info("VRaptor initializing ...");

		long start = System.currentTimeMillis();
		ServletContext appContext = app.getApplicationContext().getServletContext();
		app.init();
		Controller controller = new DefaultController(app);
		appContext.setAttribute("webApplication", app);
		appContext.setAttribute(WebApplication.class.getName(), app);
		appContext.setAttribute(ViewManager.class.getName(), app.getDefaultViewManager());
		appContext.setAttribute(Introspector.class.getName(), app.getIntrospector());
		try {
			app.getPluginManager().init(app);
		} catch (VRaptorException e) {
			throw new ConfigException("Unable to startup some plugin.", e);
		}
		long buildTime = System.currentTimeMillis() - start;

		LOG.info("VRaptor init time " + (buildTime / 1000.0) + " seconds");
		return controller;
	}

}
