package org.vraptor.http;

import java.util.Enumeration;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.vraptor.LogicRequest;
import org.vraptor.introspector.Introspector;

/**
 * A simple request wrapper capable of registering new request parameters.
 * It is also responsible for dealing with variable lookup.
 *
 * @author Guilherme Silveira
 * @author Fabio Kung
 */
public class VRaptorServletRequest extends HttpServletRequestWrapper {

	private final RequestParameters parameters;

	/**
	 * Constructor for this wrapper
	 *
	 * @param request to be wrapped
	 */
	@SuppressWarnings("unchecked")
	public VRaptorServletRequest(HttpServletRequest request) {
		super(request);
		this.parameters = new RequestParameters(request.getParameterMap());
	}

	public void setParameter(String name, String value) {
		this.parameters.put(name, value);
	}

    public void addParameterValue(String name, String value) {
        parameters.addValueToArray(name, value);
    }

    @Override
	public String getParameter(String name) {
		return parameters.getParameter(name);
	}

	@Override
	public Map getParameterMap() {
		return parameters.getParameterMap();
	}

	@Override
	public Enumeration getParameterNames() {
		return parameters.getParameterNames();
	}

	@Override
	public String[] getParameterValues(String name) {
		return parameters.getParameterValues(name);
	}

}
