/**
 * 
 */
package org.vraptor.interceptor.multipart;

import java.io.File;
import java.util.List;

import org.apache.commons.fileupload.FileItem;
import org.apache.log4j.Logger;
import org.vraptor.http.VRaptorServletRequest;
import org.vraptor.interceptor.BasicUploadedFileInformation;
import org.vraptor.interceptor.UploadedFileInformation;

public class MultipartItemsProcessor {
	private static final Logger LOG = Logger.getLogger(MultipartItemsProcessor.class);
	private final List<FileItem> items;
	private final VRaptorServletRequest request;

	public MultipartItemsProcessor(List<FileItem> items, VRaptorServletRequest request) {
		this.items = items;
		this.request = request;
	}
	
	public void process() {
		for (FileItem item : items) {
			if (item.isFormField()) {
				request.addParameterValue(item.getFieldName(), item.getString());
			}
			else {
				if (!item.getName().trim().equals("")) {
					try {
						File file = File.createTempFile("raptor.", ".upload");
						file.deleteOnExit();
						item.write(file);
						UploadedFileInformation fileInformation = new BasicUploadedFileInformation(file, item.getName(), item
								.getContentType());
						request.setAttribute(item.getFieldName(), fileInformation);
						LOG.info("Uploaded file: " + item.getFieldName() + " with " + fileInformation);
					}
					catch (Exception e) {
						LOG.error("Nasty uploaded file " + item.getName(), e);
					}
				}
				else {
					LOG.info("A file field was empy: " + item.getFieldName());
				}
			}
		}
	}
}