package org.vraptor.plugin.groovy;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.InvalidComponentException;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicMethodFactory;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.webapp.DefaultComponentManager;

/**
 * 
 * @author Nico Steppat
 */
public class GroovyComponentManager implements ComponentManager {

	private static final Logger LOG = Logger.getLogger(GroovyComponentManager.class);

	private DefaultComponentManager manager;

	public GroovyComponentManager(LogicMethodFactory factory) {
		manager = new DefaultComponentManager(factory);
	}

	public ComponentType getComponent(String name, String logic) throws ComponentNotFoundException,
			LogicNotFoundException {
		return this.manager.getComponent(name, logic);
	}

	public ComponentType getComponentType(Class<?> type) throws InvalidComponentException {
		return this.manager.getComponentType(type);
	}

	public Set<ComponentType> getComponents() {
		return this.manager.getComponents();
	}

	public ConcurrentMap<String, ConcurrentMap<String, ComponentType>> getComponentsAsMap() {
		return this.manager.getComponentsAsMap();
	}

	public void register(ComponentType type) throws LogicNotFoundException {
		LOG.debug(String.format("Registering component %s as %s", type.getComponentClass(), type.getName()));
		//always register even if already present
		this.manager.getComponentsAsMap().put(type.getName(), new ConcurrentHashMap<String, ComponentType>());
		for (LogicMethod logic : type.getLogics()) {
			this.manager.getComponentsAsMap().get(type.getName()).put(logic.getName(), type);
		}
	}

	public boolean register(String typeClazz) throws InvalidComponentException {
		return this.manager.register(typeClazz);
	}

}