package org.vraptor.plugin.jpa;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.component.BeanConstructor;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.introspector.BeanProvider;

import javax.persistence.EntityManager;
import java.lang.reflect.Field;
import java.text.MessageFormat;

/**
 * Wraps the standard {@link BeanConstructor} injecting an {@link EntityManager}
 * at the construction time.
 *
 * @author Fabio Kung
 * @since 2.3.2
 */
public class EntityManagerInjector implements BeanConstructor {

    private final BeanConstructor beanConstructor;
    private final Field entityManagerField;

    private static final Logger LOG = Logger.getLogger(EntityManagerInjector.class);

    public EntityManagerInjector(BeanConstructor constructor, Field entityManagerField) {
        this.beanConstructor = constructor;
        this.entityManagerField = entityManagerField;
    }

    public Object newInstance(LogicRequest request, BeanProvider provider) throws ComponentInstantiationException {
        Object instance = beanConstructor.newInstance(request, provider);
        injectPersistenceContext(request, instance);

        return instance;
    }

    private void injectPersistenceContext(LogicRequest request, Object component)
            throws ComponentInstantiationException {
        if (entityManagerField != null) {
            EntityManager entityManager = (EntityManager) request.getRequestContext()
                    .getAttribute(EntityManager.class.getName());
            try {
                LOG.debug(MessageFormat.format("Injecting EntityManager in the component: {0}.{1}",
                        entityManagerField.getType().getName(), entityManagerField.getName()));
                entityManagerField.setAccessible(true);
                entityManagerField.set(component, entityManager);
            } catch (IllegalAccessException e) {
                throw new ComponentInstantiationException("Problem injecting the EntityManager", e);
            }
        }
    }

}
