package org.vraptor.plugin.niceurls;

import java.util.Arrays;

import org.vraptor.LogicException;
import org.vraptor.annotations.Remotable;
import org.vraptor.component.LogicMethod;
import org.vraptor.http.VRaptorServletRequest;
import org.vraptor.plugin.niceurls.resolver.URLData;
import org.vraptor.url.InvalidURLException;
import org.vraptor.url.ViewLocator;
import org.vraptor.view.RemoteView;
import org.vraptor.view.RemoteViewManager;
import org.vraptor.view.ViewManager;

import static java.util.Arrays.asList;

public class NiceURLViewLocator implements ViewLocator {

	public ViewManager locate(VRaptorServletRequest req, LogicMethod method, ViewManager defaultManager)
			throws InvalidURLException, LogicException {
		URLData urlData = urlDataInRequest(req);
		
		boolean hasRemoteViewType = asList("xml", "ajax").contains(urlData.getViewType()); 
		
		return hasRemoteViewType ? locateRemoteManager(method, defaultManager, urlData) : defaultManager;
	}

	private URLData urlDataInRequest(VRaptorServletRequest req) {
		return (URLData)req.getAttribute(NiceURLFilter.URLDATA_ATTRIBUTE);
	}

	private ViewManager locateRemoteManager(LogicMethod method, ViewManager defaultViewManager, URLData result)
			throws LogicException {
		if (!method.getMetadata().isAnnotationPresent(Remotable.class)) {
			throw new LogicException("logic method is not @Remotable");
		}
		return new RemoteViewManager(defaultViewManager, RemoteView.valueOf(result.getViewType().toUpperCase()));
	}

}
