package org.vraptor.plugin.niceurls.component;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.annotations.Component;
import org.vraptor.annotations.Logic;
import org.vraptor.annotations.Viewless;

/**
 * Implements some redirect logics  
 */
@Component("VRaptorRedirector")
public class RedirectComponent {
	
	private final static Logger logger = Logger.getLogger(RedirectComponent.class);
	
	private HttpServletRequest request;
	private HttpServletResponse response;

	public RedirectComponent(HttpServletRequest request, HttpServletResponse response) {
		this.request = request;
		this.response = response;
	}

	/**
	 * Send redirect with Moved Temporarily Header
	 * @param uri absolute uri to redirect 
	 * @throws IOException
	 */
	@Viewless
	@Logic(parameters={"uri"})
	public void movedTemporarily(String uri) throws IOException {
		if (logger.isDebugEnabled()) logger.debug("Redirecting to " + uri);
		this.response.sendRedirect(absoluteURL(uri));
	}

	/**
	 * Send redirect with Moved Permanently Header
	 * @param uri absolute uri to redirect 
	 * @throws IOException
	 */
	@Viewless
	@Logic(parameters={"uri"})
	public void movedPermanently(String uri) throws IOException {
		if (logger.isDebugEnabled()) logger.debug("Redirecting to " + absoluteURL(uri));
		this.response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
		this.response.addHeader("Location", absoluteURL(uri));
		this.response.addIntHeader("Content-length", 0);
		this.response.addDateHeader("Date", System.currentTimeMillis());
	}

	/**
	 * Redirect request direct to view, without logic execution.
	 * Uses RequestDispatcher.
	 * @param uri 
	 * @throws ServletException
	 * @throws IOException
	 */
	@Viewless
	@Logic(parameters={"uri"})
	public void redirectToView(String uri) throws ServletException, IOException {
		this.request.getRequestDispatcher(uri).forward(request, response);
	}
	
	private String absoluteURL(String uri) {
		if (uri.startsWith("/"))
			return this.request.getContextPath() + uri;
		else
			return uri;
	}
}
