package org.vraptor.plugin.niceurls.parser;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Scanner;

import org.apache.log4j.Logger;
import org.vraptor.plugin.niceurls.resolver.Route;
import org.vraptor.plugin.niceurls.resolver.URLResolver;

/**
 * Parses a .routes file and adds routes to an URLResolver
 */
public class DefaultRoutesParser implements RoutesParser {

	private final static Logger logger = Logger.getLogger(DefaultRoutesParser.class);
	private final URLResolver resolver;
	private final FileSystem fileSystem;

	public DefaultRoutesParser(URLResolver resolver, FileSystem fileSystem) {
		this.resolver = resolver;
		this.fileSystem = fileSystem;
	}

	public void parseFile(String fileName) throws FileNotFoundException, IOException {
		InputStream file = fileSystem.open(fileName);
		
		if (logger.isDebugEnabled())
			logger.debug("Parsing routes file ");

		Scanner s = new Scanner(file);

		try {
			while (s.hasNextLine()) {
				parseLine(s.nextLine().trim());
			}
		}
		finally {
			s.close();
		}
	}

	private boolean isComment(String line) {
		return line.length() == 0 || line.charAt(0) == '#';
	}

	private void parseLine(String line) throws IOException {
		if (line.startsWith("#include")) {
			String otherFile = line.substring("#include".length()).trim();
			parseFile(otherFile);
			return;
		}
		
		if (isComment(line)) return;
		
		for (RouteDefinition rd : RouteDefinition.values()) {
			if (line.indexOf(rd.getSeparator()) > 0) {
				
				String[] split = line.split("\\s*" + rd.getSeparator()	+ "\\s*");
				resolver.addRoute(new Route(split[0], split.length == 1? null: split[1], rd.getRouteConsequence()));

				if (logger.isTraceEnabled())
					logger.trace("Route found: " + split[0] + " to " + (split.length == 1? "": split[1]));
				break;
			}
		}
	}

}



