package org.vraptor.plugin.niceurls.parser;

import org.vraptor.plugin.niceurls.consequences.DynamicRouteConsequence;
import org.vraptor.plugin.niceurls.consequences.IgnoreRouteConsequence;
import org.vraptor.plugin.niceurls.consequences.MovedPermanentlyRouteConsequence;
import org.vraptor.plugin.niceurls.consequences.RedirectRouteConsequence;
import org.vraptor.plugin.niceurls.consequences.RouteConsequence;
import org.vraptor.plugin.niceurls.consequences.SkipToViewRouteConsequence;

/**
 * Represents a Route defined by the user. Essentially we have many types of
 * routes, with different syntaxes and different Consequences;
 */
public enum RouteDefinition {
	
	// order matters!! be careful
	IGNORE(">>!", new IgnoreRouteConsequence()),
	MOVED_PERMANENTLY (">>>", new MovedPermanentlyRouteConsequence()), 
	REDIRECT (">>", new RedirectRouteConsequence()), 
	SKIP_TO_VIEW ("==>", new SkipToViewRouteConsequence()),
	LOGIC ("=>", new DynamicRouteConsequence()),;

	private final String separator;
	private final RouteConsequence routeConsequence;
	
	RouteDefinition(String s, RouteConsequence rc) {
		this.separator = s;
		this.routeConsequence = rc;
	}
	
	public String getSeparator() {
		return separator;
	}
	
	public RouteConsequence getRouteConsequence() {
		return routeConsequence;
	}
}
