package org.vraptor.plugin.niceurls.resolver;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

import org.apache.log4j.Logger;

/**
 * Default implementation to a URLResolver
 */
public class DefaultURLResolver implements URLResolver {

	private final static Logger logger = Logger.getLogger(DefaultURLResolver.class);
	private final List<Route> routes;
	
	public DefaultURLResolver() {
		this.routes = new ArrayList<Route>();
	}
	
	public void addRoute(Route route) {
		this.routes.add(route);
	}

	public URLData resolveURL(String url) throws UnknownURLException {
		if (logger.isTraceEnabled()) logger.trace("Trying to resolve " + url);
		
		for (Route route : this.routes) {
			Matcher m = route.getFromPattern().matcher(url);
			if (m.matches()) {
				
				ParamsContext context = new ParamsContext();
				// parse variables
				int i = 1;
				for (String param : route.getParamNames()) {
					context.addParameter(param, m.group(i++));
				}

				URLData result = route.generateResult(context);
				if (result != null)
					result.setParamsContext(context);
				if (logger.isTraceEnabled()) logger.trace("Result found: " + result);
				return result;
			}
		}

		throw new UnknownURLException("NiceURLs URLResolver could not resolve url", url);
	}
	
	public List<Route> getRoutes() {
		return routes;
	}


}
