package org.vraptor.url;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.http.VRaptorServletRequest;

/**
 * The default url manager translates url data.
 * 
 * @author Guilherme Silveira
 */
public class DefaultLogicLocator implements LogicLocator {

	private static final Logger LOG = Logger.getLogger(DefaultLogicLocator.class);

	private static final String INCLUDE_REQUEST_URI = "javax.servlet.include.request_uri";

	private final ComponentManager manager;
	
	public DefaultLogicLocator(ComponentManager manager) {
		this.manager = manager;
	}

	public LogicMethod locate(VRaptorServletRequest req) throws InvalidURLException, LogicNotFoundException, ComponentNotFoundException {
        String uri = extractLogicInfoFromRequestURI(req);

		int firstPosition = uri.indexOf('.');
		if (firstPosition == -1)
			throw new InvalidURLException("Unable to deal with desired url: " + uri);

		int secondPosition = uri.indexOf('.', firstPosition + 1);
		if (secondPosition == -1)
			throw new InvalidURLException("Unable to deal with desired url: " + uri);

		String componentName = uri.substring(0, firstPosition);
		String actionName = uri.substring(firstPosition + 1, secondPosition);

		ComponentType component = manager.getComponent(componentName, actionName);
		LogicMethod method = component.getLogic(actionName);
		
		if (LOG.isDebugEnabled()) {
			LOG.debug("logic method extracted: " + method);
		}
		return method;
		
	}

    String extractLogicInfoFromRequestURI(HttpServletRequest req) {
        String uri = extractURI(req);
        if (LOG.isDebugEnabled()) {
            LOG.debug("Requested url (request wrapped): " + uri);
        }

        String contextPath = req.getContextPath();
        int startPosition = uri.indexOf(contextPath) + contextPath.length() + 1;

        uri = uri.substring(startPosition);
        if (LOG.isDebugEnabled()) {
            LOG.debug("requested uri: " + uri);
        }
        return uri;
    }

    private String extractURI(HttpServletRequest req) {
		String uri;
		// details in : http://www.caucho.com/resin-3.0/webapp/faq.xtp#forward-path
		// and http://www.javaworld.com/javaworld/jw-03-2003/jw-0328-servlet.html?page=4
		if (req.getAttribute(INCLUDE_REQUEST_URI) != null) {
			uri = (String) req.getAttribute(INCLUDE_REQUEST_URI);
		} else {
			uri = req.getRequestURI();
		}

		return uri;
	}

}
