package org.vraptor.url;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.vraptor.LogicException;
import org.vraptor.http.VRaptorServletRequest;
import org.vraptor.annotations.Remotable;
import org.vraptor.component.LogicMethod;
import org.vraptor.view.RemoteView;
import org.vraptor.view.RemoteViewManager;
import org.vraptor.view.ViewManager;

public class DefaultViewLocator implements ViewLocator {

	private static final Logger LOG = Logger.getLogger(DefaultViewLocator.class);

	private static final String INCLUDE_REQUEST_URI = "javax.servlet.include.request_uri";

	public ViewManager locate(VRaptorServletRequest req, LogicMethod method, ViewManager defaultViewManager)
			throws InvalidURLException, LogicException {
		String uri = extractURI(req);

		if (LOG.isDebugEnabled()) {
			LOG.debug("Requested url (request wrapped): " + uri);
		}

		uri = uri.substring(uri.lastIndexOf('/') + 1);
		if (LOG.isDebugEnabled()) {
			LOG.debug("requested uri: " + uri);
		}

		int firstPosition = uri.indexOf('.');
		if (firstPosition == -1)
			throw new InvalidURLException("Unable to deal with desired url: " + uri);

		int secondPosition = uri.indexOf('.', firstPosition + 1);
		if (secondPosition == -1)
			throw new InvalidURLException("Unable to deal with desired url: " + uri);

		int thirdPosition = uri.indexOf('.', secondPosition + 1);
		String viewType = null;
		if (thirdPosition != -1) {
			viewType = uri.substring(secondPosition + 1, thirdPosition);
		}
		
		//TODO allow view managers to be expanded by registering your own managers...
		// this code should look go for each one as in the converter finder
		if (("xml".equals(viewType) || "ajax".equals(viewType))) {
			if(!method.getMetadata().isAnnotationPresent(Remotable.class)) {
				throw new LogicException("logic method is not @Remotable");
			}
			return new RemoteViewManager(defaultViewManager, RemoteView.valueOf(viewType.toUpperCase()));
		}
		
		return defaultViewManager;
	}

	private String extractURI(HttpServletRequest req) {
		String uri;
		// details in : http://www.caucho.com/resin-3.0/webapp/faq.xtp#forward-path
		// and http://www.javaworld.com/javaworld/jw-03-2003/jw-0328-servlet.html?page=4
		if (req.getAttribute(INCLUDE_REQUEST_URI) != null) {
			uri = (String) req.getAttribute(INCLUDE_REQUEST_URI);
		} else {
			uri = req.getRequestURI();
		}

		return uri;
	}

}
